function mlogL = minusloglikelihood(vec)

% The following function calculates the opposite (because matlab only has 
% minimization routines, i.e., no maximization routine) of the model 
% log-likelihood according to the model parameters (vector 'vec' in this function)

% First we begin by importing as global variables some variables defined in 
% the 'run_flucovariates.m' file (because they could be defined once and for all)
% and that will be used in the calculation of the minus log-likelihood:

global yearornot regionornot nbX nbR nbY indyear1 indyear2 indregion1 indregion2 Xbound1 Xbound2 INCbound1 INCbound2


%%%%%%%%%%%%%%%%%%%%
% Model parameters %
%%%%%%%%%%%%%%%%%%%%

% The parameter vector ('vec') is organized as follows:
% The first value is the logarithm of the exponent of the flow of virus entry.
% The second value is the logarithm of the coefficient associated with the
% local transmission term (global incidence)
% values from 3 to nbX represent the coefficients 
% associated to quantitative cofactors: (mobility flows have no associated parameter)

expo = vec(1); % the exponent of the virus entry flow estimate phi
aINC = vec(2); % the logarithm of the coefficient associated to global transmission
aX = vec(3:nbX); 

% if yearornot = 1, the subsequent nbY values represent the 'year'
% associated coefficients: (if yearornot = 0, ayear = 0)

ayear = [0, vec((nbX+1):(nbX+yearornot*nbY))]; 
% Note that '0' is bound on the left of the 'ayear' vector because the 
% first year is the reference year 

% if regionornot = 1, the subsequent nbR values represent the 'region'
% associated coefficients: (if regionornot = 0, aregion = 0)

aregion = [0, vec((nbX+yearornot*nbY+1):(nbX+yearornot*nbY+regionornot*nbR))];

% Note that '0' is bound on the left of the 'aregion' vector because the 
% first region is the reference region 

% The last value of the parameter vector allways represents the intercept
% of the model:
aInt = vec(nbX+yearornot*nbY+regionornot*nbR + 1); 
       

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculation of the lambdas %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% in the model, the opposite of the logarithm of the likelihood (variable
% 'mlogL' in this function) is given by the difference between two terms 
% (see the main text of the paper for mathematical justification):
% mlogL = sum(other all regions R) of the sum(other all years Y) of the sum(for week w between week 0 and the week before the epidemic week) of lambda(w, R, Y)
%       - sum(other all regions R) of the sum(other all years Y) of log(1 - exp(lambda(epidemic week,R,Y)

% In the main function ('run_flucovariates.m'), we defined a matrix 'Xbound1' that binds
% the values of each quantitative cofactor used in the model 
% (one line = one cofactor) for all regions, all years and all values 
% of w between 0 and the epidemic week minus one. Xbound1 is used to calculate the 
% following vector:

logbeta1 = aInt + aX * Xbound1;

% This vector (almost) binds all the values of the logarithm of lambda for all
% regions, all years and all values of w between 0 and the epidemic week.

% The same calculation is done to calculate the vector that (almost) binds 
% all the values of the logarithme of lambda for all regions and all years 
% for the epidemic week (Xbound2 being the corresponding cofactor matrix):

logbeta2 = aInt + aX * Xbound2;

% Since the model may also incorporate additive 'year' and 'region'
% effects (not done in our analysis), the following manipulation is done:

if yearornot == 1
    logbeta1 = logbeta1 + ayear(indyear1);
    logbeta2 = logbeta2 + ayear(indyear2);
end

if regionornot == 1
    logbeta1 = logbeta1 + aregion(indregion1);
    logbeta2 = logbeta2 + aregion(indregion2);
end

% To avoid NA (such as expo(large number) or log(0)), we constrained the logarithm
% of beta to be between -10 and +10:
logbeta1 = min(max(logbeta1, -10),10);
logbeta2 = min(max(logbeta2, -10),10);

% lambdas are simply the exponential of logbetas multiplied by the incidence term:
lambda1 = exp(logbeta1) .* ((([1, exp(aINC)] * INCbound1)).^exp(expo));
lambda2 = exp(logbeta2) .* ((([1, exp(aINC)] * INCbound2)).^exp(expo));




%%%%%%%%%%%%%%%%%%%%%
% mlogL calculation %
%%%%%%%%%%%%%%%%%%%%%

% Once lambda1 and lambda2 are calculated, the calculation of mlogL is
% quite straightforward:

mlogL = sum(lambda1) - sum(log(1-exp(-lambda2)));





