function [mlogL, vec] = run_flucovariates(mat_onset, mat_flow, mat_incidence, mat_climate, mat_demography, RoN, YoN, cofind, permut_ind, vec0)


%%%%%%%%%%%%%%%%%%%%
% function entries %
%%%%%%%%%%%%%%%%%%%%

% mat_onset: a matrix giving information about the observed epidemic onset 
% dates. The matrix has three columns :
% 1) Index of the region:
%   1 = Alsace
%   2 = Aquitaine
%   3 = Auvergne
%   4 = Basse-Normandie
%   5 = Bourgogne
%   6 = Bretagne
%   7 = Centre
%   8 = Champagne-Ardennes
%   9 = Haute-Normandie
%   10 = Ile de France
%   11 = Lorraine
%   12 = Midi-Pyrenees
%   13 = Nord-Pas-de-Calais
%   14 = Pays-de-la-Loire
%   15 = Picardie
%   16 = Poitou-Charentes
%   17 = Provence-Alpes-Cote-d-Azur
%   18 = Rhone-Alpes
% 
% 2) index of the epidemic year:
%   1 = 2006-2007
%   2 = 2007-2008
%   3 = 2008-2009
%   4 = 2010-2011
%   5 = 2011-2012
%
% 3) estimated week of the epidemic onset in the region during the epidemic year. 
% Week 0 in the model corresponds to week 41 in the calendar.
% Any negative number will be interpreted by the program as an absence of
% observed epidemic year in the corresponding region during the
% corresponding epidemic year.



% mat_flow: a matrix representing the estimated mobility flows between each 
% pair of the 18 studied regions (same numbering as above for regions).

% mat_incidence: The second matrix involved in the estimation of the mobility 
% flow of infected people. This part concerns incidence. The matrix has four columns:
% 1) region index
% 2) epidemic year index
% 3) week index of the associated region and epidemic year 
% (week 0 corresponds to week 41 in the calendar)
% 4) corresponding value of the ratio I/N = incidence divided by population
% size.


% mat_climate: a matrix giving information about the estimated value of the 
% climate cofactors. The matrix has five columns:
% 1) region index
nbregions = max(mat_climate(:,1)); % = 18 = the number of regions in the data set
% 2) epidemic year index
nbyears = max(mat_climate(:,2)); % = 6 = the number of (epidemic) years in the data set
% 3) week index of the associated region and epidemic year 
nbweeks = max(mat_climate(:,3)); % = 25 = the number of weeks during each epidemic year
% (week 0 corresponds to week 41 in the calendar)
% 4) corresponding value of humidity (centered and reduced)
% 5) corresponding value of temperature (centered and reduced)


% mat_demography: a matrix giving information about the estimated values of the 
% demographic cofactors. The matrix has three columns:
% 1) region index
% 2) average proportion of children in the region (centered and reduced)
% 3) log of the average number of individuals in the region (centered and
% reduced)

% Variable 'RoN', which exactly equals the variable 'regionornot', and tells 
% whether the 'year' cofactor is included in the model (as a qualitative
% variable). The difference between both variables is that RoN is a function 
% entry, whereas regionornot is a global variable (distinction is made for 
% computational constrains). RoN=1 means that the 'region' cofactor is considered 
% in the model. Otherwise (here RoN = 0) it is not (which is the choice we made 
% our analysis).
global regionornot
regionornot = RoN;

% The same principle applies to the 'year' cofactoir.
% Variable 'YoN', which exactly equals the variable 'yearornot', and tells 
% whether the 'year' cofactor is included in the model (as a qualitative
% variable). The difference between both variables is that RoN is a function 
% entry, whereas yearornot is a global variable (distinction is made for 
% computational constrains). YoN=1 means that the 'year' cofactor is considered 
% in the model. Otherwise (here YoN = 0) it is not (which is the choice we made 
% our analysis)
global yearornot
yearornot = YoN;

% The entry variable 'cofind' stands for 'cofactor indexes'. It is a vector
% that binds all the indexes of the cofactors that are considered.
% Potentially considered cofactors are:
% 1) mobility flows
% 2) total prevalence over all regions
% 3) log of the Region size
% 4) Proportion of children in the region
% 5-9) Humidity cofactors:
%   5)  Humidity, as it is given in the mat_climate matrix.
humidity = mat_climate(:,4);

%   6)  HW is the weekly component of the variations of humidity.
%       It represents the variation between weeks in the average  
%       (over all years and all regions) value of humidity.

for week = 1:nbweeks
    
    HW(1,week) = mean(mat_climate(find(mat_climate(:,3)==week),4));
    
end


%   7)  HR is the regional component of the variations of humidity.
%       It represents the variation between regions in the average  
%       (over all years and all weeks) value of humidity.

for region = 1:nbregions
    
    HR(1,region) = mean(mat_climate(find(mat_climate(:,1)==region),4));
    
end

%   8)  HR is the annual component of the variations of humidity.
%       Its definition is a bit more complexe, see the main text for more
%       details.

for year = 1:nbyears
    for week = 1:nbweeks

        HY(year,week) = mean(mat_climate(find((mat_climate(:,3) == week) & (mat_climate(:,2) == year)),4)) - HW(1,week);
        
    end
end

%   9) Hres stands for residual variations in humidity:

for year = 1:nbyears
    for week = 1:nbweeks
        for region = 1:nbregions

            Hres(region,week, year) = mat_climate(find((mat_climate(:,2)==year) & (mat_climate(:,3) == week) & (mat_climate(:,1) == region)),4) - HW(1,week) - HR(1, region) - HY(year,week);
            
        end
    end
end

            
% 10-14) temperature cofactors. The same decomposition applies:
%   10)  temperature, as it is given in the mat_climate matrix.
temperature = mat_climate(:,5);

%   11)  TW is the weekly component of the variations of temperature.
%       It represents the variation between weeks in the average  
%       (over all years and all regions) value of temperature.
for week = 1:nbweeks
    
    TW(1,week) = mean(mat_climate(find(mat_climate(:,3)==week),5));
    
end

%   12)  TR is the regional component of the variations of temperature.
%       It represents the variation between regions in the average  
%       (over all years and all weeks) value of temperature.
for region = 1:nbregions
    
    TR(1,region) = mean(mat_climate(find(mat_climate(:,1)==region),5));
    
end
%   13)  TR is the regional component of the variations of temperature.
%       Its definition is a bit more complexe, see the main text for more
%       details.
for year = 1:nbyears
    for week = 1:nbweeks

        TY(year,week) = mean(mat_climate(find((mat_climate(:,3) == week) & (mat_climate(:,2) == year)),5)) - TW(1,week);
        
    end
end
%   14) Tres stands for residual variations in temperature:
for year = 1:nbyears
    for week = 1:nbweeks
        for region = 1:nbregions

            Tres(region,week, year) = mat_climate(find((mat_climate(:,2)==year) & (mat_climate(:,3) == week) & (mat_climate(:,1) == region)),5) - TW(1,week) - TR(1, region) - TY(year,week);
            
        end
    end
end


% Each model is defined by the set of cofactors that are included, i.e., by 
% the value of variables RoN, YoN and cofind. For example, if RoN=1, YoN=0
% and cofind = [1 2 3 5 10 11 12 13] will be a model with 'region',
% 'mobility flows', 'total incidence', 'region size', 'humidity' and the
% weekly, regional, annual and residual components of 'temperature'

% 'permut_ind' is a vector made up of the indexes of all the variables that 
% are permuted in the simulation. Usually, in permutation tests, only one 
% variable is randomised so permut_ind has rarely more than one element. 
if length(permut_ind) > 0
    randomize % calls the randomize file when at least one cofactor is permuted
end

% vec0 is the initial value taken for the vector of the model parameters

% The last preliminary calculation consists in defining the global
% variables that will be used in the minusloglikelihood fuinction (see the 
% function for the definition of these variables):

global nbX nbR nbY indyear1 indyear2 indregion1 indregion2 Xbound1 Xbound2 INCbound1 INCbound2

nbX = length(cofind);
nbR = nbregions - 1;
nbY = nbyears - 1;
% the calculation of indyear1, indyear2, indregion1, indregion2, Xbound1 
% and Xbound2 will be performed below.



% The log-likelihood is the sum of the logarithm of the probability of 
% all the observed epidemic onset dates (or the absence of) on all regions
% and all epidmic years. We begin here by considering a for loop on all the 
% lines of the mat_onset variable, which will scan, line by line, the different 
% epidemic onset dates that have been observed.
% From the observed epidemic onset dates, we will build four matrices,
% Xbound1, Xbound2, INCbound1 and INCbound2:
% - 'Xbound1' binds the values of each quantitative cofactor used in the model 
% (one line = one cofactor) for all regions, all years and all values 
% of w between 0 and the epidemic week minus one.
% - 'Xbound2' binds the values of each quantitative cofactor used in the model 
% (one line = one cofactor) for all regions and all years during the exact 
% epidemic week.
% - 'INCbound1' binds the values of the two quantitative cofactor
% corresponding to the incidence fonction (first line = local transmission,
% second line = global transmission) for all regions, all years and all values 
% of w between 0 and the epidemic week minus one.
% - 'INCbound2' binds the values of each quantitative cofactor used in the model 
% (one line = one cofactor) for all regions and all years during the exact 
% epidemic week.

% INCbound2, indyear1, indyear2, indregion1, indregion2 contain the
% corresponding values of region and year for the Xbound1/INCbound1 and
% Xbound1/INCbound1 matrices

Xbound1 = [];
Xbound2 = [];
INCbound1 = [];
INCbound2 = [];
indyear1 = [];
indyear2 = [];
indregion1 = [];
indregion2 = [];

for k = 1:length(mat_onset(:,1)) % We scan all lines (=observed epidemic onsets) of the mat_onset matrix 
    region = mat_onset(k,1); % region in which the epidemic occured
    year = mat_onset(k,2); % year during which the epidemic occured
    EOD = mat_onset(k,3); % EOD stands for 'epidemic onset date' (EO<0 => no observed epidemic)
    
    % Xbound1_0, Xbound2_0, INCbound1_0 and INCbound2_0 are the values of
    % Xbound1, Xbound2, INCbound1 and INCbound2 corresping to this
    % particular observed epidemic
    
    Xbound1_0 = [];
    Xbound2_0 = [];
    INCbound1_0 = [];
    INCbound2_0 = [];
    
    for index = 1:length(cofind) % scans all the indexes of cofactors that are included in the model
       cof = cofind(index); % considered cofactor
       switch cof
           case 1 % local transmission
               for week = 1:nbweeks
                   inc_vec = mat_incidence(find((mat_incidence(:,2)==year) & (mat_incidence(:,3) == week)),4); % a vector gathering for all regions the incidence at year 'year' during week 'week' 
                   cofvec(1,week) = (mat_flow(region, :) + mat_flow(:,region)') * inc_vec; % For week 'week', the product of all flow (one way or the other) between the region considered and all others times the incidence of other regions
               end
           case 2 % global transmission
               for week = 1:nbweeks
                   cofvec(1,week) =  sum(mat_incidence(find((mat_incidence(:,2)==year) & (mat_incidence(:,3) == week)),4)); % the sum of the incidences at year 'year' during week 'week' other all regions 
               end           
           case 3 % log-population size
               cofvec = mat_demography(region, 3) * ones(1,max(mat_incidence(:,3))); % repeats the value of the log population size a number of times egals to the number of weeks
           case 4 % proportion of children
               cofvec = mat_demography(region, 2) * ones(1,max(mat_incidence(:,3))); % repeats the value of the proportion of children a number of times egals to the number of weeks
           case 5 % humidity
               for week = 1:nbweeks
                   cofvec(1,week) = humidity(find((mat_climate(:,2)==year) & (mat_climate(:,3) == week) & (mat_climate(:,1) == region))); % values of 'humidity' for all weeks in region 'region' and for year 'year' 
               end               
           case 6 % humidity (weekly variations)
               cofvec = HW;
           case 7 % humidity (regional variations)
               cofvec = HR(1,region) * ones(1,max(mat_incidence(:,3)));
           case 8 % humidity (inter-annual variations)
               cofvec = HY(year,:);
           case 9 % humidity (residual = spatio-temporal variations)
               cofvec = Hres(region,:,year);
           case 10 % temperature
               for week = 1:nbweeks
                   cofvec(1,week) = temperature(find((mat_climate(:,2)==year) & (mat_climate(:,3) == week) & (mat_climate(:,1) == region))); % values of 'temperature' for all weeks in region 'region' and for year 'year' 
               end
           case 11 % temperature (weekly variations)
               cofvec = TW;
           case 12 % temperature (regional variations)
               cofvec = TR(1,region) * ones(1,max(mat_incidence(:,3)));
           case 13 % temperature (inter-annual variations)
               cofvec = TY(year,:);
           case 14 % temperature (residual = spatio-temporal variations)
               cofvec = Tres(region,:,year);
       end
       
       % step by step construction of Xbound1_0, Xbound2_0, INCbound1_0 and 
       % INCbound2_0 from the binding of the corresponding value for each cofactor 
       if EOD > 0 % if an epidemic was observed
           if index > 2
               Xbound1_0 = [Xbound1_0; cofvec(1:(EOD-1))];
               Xbound2_0 = [Xbound2_0; cofvec(EOD)];
           else 
               INCbound1_0 = [INCbound1_0; cofvec(1:(EOD-1))];
               INCbound2_0 = [INCbound2_0; cofvec(EOD)];
           end
       else % no epidemic
           if index > 2
               Xbound1_0 = [Xbound1_0; cofvec(1:end)];
           else
               INCbound1_0 = [INCbound1_0; cofvec(1:end)];
           end
       end
       
    end
    
    % step by step construction of Xbound1, Xbound2, INCbound1 and INCbound2
    % from the binding of Xbound1_0, Xbound2_0, INCbound1_0 and INCbound2_0 
    % for all observed epidemics
    Xbound1 = [Xbound1, Xbound1_0];
    Xbound2 = [Xbound2, Xbound2_0];
    INCbound1 = [INCbound1, INCbound1_0];
    INCbound2 = [INCbound2, INCbound2_0];
    
    if EOD > 0
        indyear1 = [indyear1, year * ones(1,EOD-1)];
        indyear2 = [indyear2, year];
        indregion1 = [indregion1, region * ones(1,EOD-1)];
        indregion2 = [indregion2, region]; 
    else
        indyear1 = [indyear1, year * ones(1,max(mat_incidence(:,3)))];
        indregion1 = [indregion1, region * ones(1,max(mat_incidence(:,3)))];
    end
    
end

% Finding the minimum of the minus log-likelihood function. 
% vec (the parameter vector) is initialized at vec0, then estimated from the function
% fminsearch. If minimisation fails (exitflag ~=1, mainly because the basic
% number of steps used in the routine was not sufficient), the process is repeated
% (starting from the initial value vec) until a true minimum is reached. 
vec = vec0;
exitflag = 0;
fvalold = 0;
fval = 1;
while (exitflag ~= 1) & abs(fval-fvalold) > 1e-8 

    
    [vec,fval,exitflag] = fminsearch(@minusloglikelihood, vec);
    fvalold = fval;
    
end
% minusloglikelihood
mlogL = minusloglikelihood(vec);




